/*
 * Copyright 2013 Torindo Nesci.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.flamenk;

import com.google.common.base.Preconditions;

/**
 * The article extraction algorithm tuning parameters.
 *
 * <br/><br/><b>This implementation is Thread Safe.</b>
 *
 * @author Torindo Nesci
 */
public class AlgorithmConfiguration {

  private static final AlgorithmConfiguration DEFAULT_INSTANCE =
          new AlgorithmConfiguration();
  
  private volatile int mBucketTokens =
          HistogramConstants.HIST_BUCKET_NTOKENS;
  private volatile double mMinHistBucketTextRatio =
          HistogramConstants.TEXT_TOKEN_RATIO_TH;
  private volatile int mMinHistSegmentBuckets =
          HistogramConstants.MIN_BUCKETS_PER_SEGMENT_TH;
  private volatile boolean mAllSegments = true;
  
  public AlgorithmConfiguration() {
  
  }
  
  /**
   * Returns the default configuration.
   *
   * @return The algorithm configuration.
   */
  public static AlgorithmConfiguration getDefault() {
    return DEFAULT_INSTANCE;
  }
  
  /**
   * Sets the number of tokens each histogram bucket has to contain.
   *
   * @param number The number of tokens for each bucket histogram.
   * @return This algorithm configuration.
   */
  public AlgorithmConfiguration withBucketTokens(int number) {
    Preconditions.checkArgument(number > 0);
    
    this.mBucketTokens = number;
    return this;
  }
  
  /**
   * Returns the number of tokens each histogram bucket has to contain.
   *
   * @return The number of tokens for each bucket histogram.
   */
  public int getBucketTokens() {
    return this.mBucketTokens;
  }
  
  /**
   * Sets the histogram bucket threshold. All buckets with a text to tokens ratio
   * below this threshold are filtered out.
   *
   * @param ratio The min text to tokens ration for each histogram bucket.
   * @return This algorithm configuration.
   */
  public AlgorithmConfiguration withMinBucketTextRatio(double ratio) {
    Preconditions.checkArgument(ratio > 0);
    
    this.mMinHistBucketTextRatio = ratio;
    return this;
  }
  
  /**
   * Returns the histogram bucket threshold. All buckets with a text to tokens ratio
   * below this threshold are filtered out.
   * @return The min text to tokens ration for each histogram bucket.
   */
  public double getMinBucketTextRatio() {
    return this.mMinHistBucketTextRatio;
  }
  
  /**
   * Sets the min number of buckets a histogram segment must contain,
   * all segments with a number of buckets below this threshold are filtered out.
   *
   * @param number The min number of bucket for each histogram segment.
   * @return This algorithm configuration.
   */
  public AlgorithmConfiguration withMinSegBuckets(int number) {
    Preconditions.checkArgument(number > 0);
    
    this.mMinHistSegmentBuckets = number;
    return this;
  }
  
  /**
   * Returns the min number of buckets a histogram segment must contain,
   * all segments with a number of buckets below this threshold are filtered out.
   *
   * @return The min number of bucket for each histogram segment.
   */
  public int getMinSegBuckets() {
    return this.mMinHistSegmentBuckets;
  }
  
  /**
   * If set to <tt>true</tt> all histogram segments within the max ranked block
   * are processed, otherwise only the segment with max rank.
   *
   * @param processAll The process all segments parameter.
   * @return This algorithm configuration.
   */
  public AlgorithmConfiguration withProcessAllSegments(boolean processAll) {
    this.mAllSegments = processAll;
    
    return this;
  }
  
  /**
   * Returns the process all segment parameter.
   * If set to <tt>true</tt> all histogram segments within the max ranked block
   * are processed, otherwise only the segment with max rank.
   *
   * @return The process all segment parameter.
   */
  public boolean getProcessAllSegments() {
    return this.mAllSegments;
  }
  
}
