/*
 * Copyright 2013 Torindo Nesci.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.flamenk.article.manipulators;

import com.flamenk.article.Article;
import com.flamenk.dom.HtmlNode;
import com.flamenk.dom.HtmlNodeRange;

/**
 * A node manipulator.
 * It's used to manipulate a html node during the article extraction.
 * One or more node manipulators can be registered
 * against the {@link com.flamenk.article.ArticleExtractor} in order to control
 * the output article's html. <br/>
 * All nodes that are elected as part of the article
 * are consumed by all manipulators registered against the
 * {@link com.flamenk.article.ArticleExtractor} using the
 * {@link com.flamenk.article.ArticleExtractor#addNodeManupulator(com.flamenk.article.manipulators.HtmlNodeManipulator)}
 * method. For instance, if the {@code fb-like} nodes are unwanted, they
 * can be filtered out using the following implementation:
 * <pre>
 * publc void manipulate(HtmlNode node, Article article, HtmlNodeRange maxRankRange) {
 *  if ("fb-like".equals(node.getName())) {
 *    node.setDisplayMode(HtmlNodeDisplayMode.NOT_DISPLAY);
 *  }
 * }
 * </pre>
 *
 * @author Torindo Nesci
 */
public interface HtmlNodeManipulator {

  /**
   * Manipulates the given html node.
   * The node with all its children and ancestors can be manipulated in order to
   * filter out some nodes or to extract some information. The input parameter
   * <tt>maxRankRange</tt> represents the
   * {@link com.flamenk.article.HistogramSegment} with max rank,
   * however the output article will be
   * the composition of all {@link com.flamenk.article.HistogramSegment}
   * returned by
   * {@link com.flamenk.article.HistogramSegmentExtractor#extract(com.flamenk.histogram.Histogram)}.
   *
   * @param node The html node.
   * @param article The article built so far.
   * @param maxRankRange The range in the html document with the highest
   * rank (likelihood) to be the article.
   */
  void manipulate(HtmlNode node, Article article, HtmlNodeRange maxRankRange);
  
}
