/*
 * Copyright 2013 Torindo Nesci.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.flamenk.article.manipulators;

import com.flamenk.TagConstants;
import com.flamenk.article.Article;
import com.flamenk.dom.HtmlNode;
import static com.flamenk.dom.HtmlNodeDisplayMode.NOT_DISPLAY;
import com.flamenk.dom.HtmlNodeRange;
import com.flamenk.util.Consumer;
import static com.flamenk.util.NodeUtil.hasName;
import com.google.common.base.Optional;

/**
 * A html list manipulator.
 * The html list is removed if the list link text ratio is grater than 0.9.
 * The link ratio is <tt>num_link_tokens / num_text_tokens</tt>.
 * <br/>
 * This implementation tries to find the list header, if the search succeeds
 * and the list is removed from the output, then the list header is removed as well.
 *
 * <br/><br/><b>This implementation is Thread Safe.</b>
 *
 * @see HtmlNode#getNumTextLinkTokens()
 * @see HtmlNode#getNumTextTokens()
 * @author Torindo Nesci
 */
public final class ListNodeManipulator implements HtmlNodeManipulator {

  private static final ListNodeManipulator INSTANCE =
          new ListNodeManipulator();
  private static final Consumer<HtmlNode> HIDE_NODE_CONSUMER =
          new Consumer<HtmlNode>() {
            public void consume(HtmlNode node) {
              node.setDisplayMode(NOT_DISPLAY);
            }
          };

  private ListNodeManipulator() {
    
  }

  /**
   * Returns the singleton instance of {@link ListNodeManipulator}.
   *
   * @return The instance of {@link ListNodeManipulator}.
   */
  public static ListNodeManipulator getInstance() {
    return INSTANCE;
  }

  @Override
  public void manipulate(HtmlNode node, Article article, HtmlNodeRange maxRankRange) {
    if (!hasName(node, TagConstants.LI)) {
      return;
    }
    
    Optional<HtmlNode> listNodeOpt = ListUtil.listNodeFor(node);
    if (!listNodeOpt.isPresent()) {
      return;
    }
    
    HtmlNode listNode = listNodeOpt.get();
    double linkRatio =
            listNode.getNumTextLinkTokens()
            / (double) listNode.getNumTextTokens();
    if (linkRatio > 0.9D) {
      listNode.setDisplayMode(NOT_DISPLAY);
      ListUtil.hideListHeader(listNode);
    }
  }

}
