/*
 * Copyright 2013 Torindo Nesci.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.flamenk.article.manipulators;

import static com.flamenk.TagConstants.A;
import static com.flamenk.TagConstants.APPLET;
import static com.flamenk.TagConstants.AREA;
import static com.flamenk.TagConstants.AUDIO;
import static com.flamenk.TagConstants.BUTTON;
import static com.flamenk.TagConstants.CANVAS;
import static com.flamenk.TagConstants.DIV;
import static com.flamenk.TagConstants.EMBED;
import static com.flamenk.TagConstants.FIELDSET;
import static com.flamenk.TagConstants.FORM;
import static com.flamenk.TagConstants.IFRAME;
import static com.flamenk.TagConstants.IMG;
import static com.flamenk.TagConstants.INPUT;
import static com.flamenk.TagConstants.KEYGEN;
import static com.flamenk.TagConstants.LI;
import static com.flamenk.TagConstants.LINK;
import static com.flamenk.TagConstants.MAP;
import static com.flamenk.TagConstants.OBJECT;
import static com.flamenk.TagConstants.PRE;
import static com.flamenk.TagConstants.SOURCE;
import static com.flamenk.TagConstants.SPAN;
import static com.flamenk.TagConstants.TEXTAREA;
import static com.flamenk.TagConstants.VIDEO;
import com.flamenk.article.Article;
import com.flamenk.dom.HtmlNode;
import static com.flamenk.dom.HtmlNodeDisplayMode.DISPLAY;
import static com.flamenk.dom.HtmlNodeDisplayMode.DISPLAY_ONLY_CHILDREN;
import static com.flamenk.dom.HtmlNodeDisplayMode.NOT_DISPLAY;
import com.flamenk.dom.HtmlNodeRange;
import static com.flamenk.util.NodeUtil.hasAncestor;
import static com.flamenk.util.NodeUtil.hasName;

/**
 * Filters out some objects that do not add content to the article, such as
 * empty div nodes, links with no href attribute etc.
 *
 * <br/><br/><b>This implementation is Thread Safe.</b>
 *
 * @author Torindo Nesci
 */
public final class PolishManipulator implements HtmlNodeManipulator {

  private static final PolishManipulator INSTANCE = new PolishManipulator();
  //
  private static final String ID_ATTR = "id";
  private static final String SRC_ATTR = "src";
  private static final String HREF_ATTR = "href";
  private static final String STYLE_ATTR = "style";
  private static final String DISPLAY_NONE_STYLE = "display:none";
  private static final String DISPLAY_BLOCK_STYLE = "display:block";
  private static final String ONCLICK = "onclick";
  private static final String GIF_EXT = ".gif";

  private PolishManipulator() {
    
  }

  /**
   * Returns the singleton instance of {@link PolishManipulator}.
   *
   * @return The {@link PolishManipulator} instance.
   */
  public static PolishManipulator getInstance() {
    return INSTANCE;
  }

  @Override
  public void manipulate(HtmlNode node, Article article, HtmlNodeRange maxRankRange) {
    if (hasName(node, APPLET)
            || hasName(node, AREA)
            || hasName(node, AUDIO)
            || hasName(node, BUTTON)
            || hasName(node, CANVAS)
            || hasName(node, FORM)
            || hasName(node, INPUT)
            || hasName(node, TEXTAREA)
            || hasName(node, FIELDSET)
            || hasName(node, EMBED)
            || hasName(node, KEYGEN)
            || hasName(node, MAP)
            || hasName(node, OBJECT)
            || hasName(node, SOURCE)
            || hasName(node, VIDEO)) {

      node.setDisplayMode(NOT_DISPLAY);

    } else if (hasName(node, IFRAME)) {
      node.setDisplayMode(DISPLAY_ONLY_CHILDREN);
    } else if (hasName(node, LI)
            && node.toText().trim().isEmpty()) {
      node.setDisplayMode(NOT_DISPLAY);
    } else if (hasName(node, IMG)) {
      if (node.hasAttribute(SRC_ATTR)) {
        String src = node.getAttributeValue(SRC_ATTR);
        if (src.endsWith(GIF_EXT)) {
          node.setDisplayMode(NOT_DISPLAY);
        }
      } else {
        node.setDisplayMode(NOT_DISPLAY);
      }
    } else if (hasName(node, A)
            && !node.hasAttribute(HREF_ATTR)) {
      node.setDisplayMode(DISPLAY_ONLY_CHILDREN);
    } else if (hasName(node, DIV)) {
      node.setDisplayMode(DISPLAY_ONLY_CHILDREN);
    } else if (hasName(node, LINK)) {
      node.setDisplayMode(NOT_DISPLAY);
    } else if (hasName(node, SPAN)
            && node.getNumTextTokens() == 0
            && !hasAncestor(node, PRE)) {
      node.setDisplayMode(NOT_DISPLAY);
    }
   
    if (node.hasAttribute(STYLE_ATTR)) {
      String style = node.getAttributeValue(STYLE_ATTR).replaceAll(" ", "");
      if (style.contains(DISPLAY_NONE_STYLE)) {
        node.setDisplayMode(NOT_DISPLAY);
      } else {
        node.removeDisplayAttribute(STYLE_ATTR);
      }
    }

    if (hasName(node, IMG) && node.getDisplayMode() == DISPLAY) {
      node.setDisplayAttribute(STYLE_ATTR, DISPLAY_BLOCK_STYLE);
    }
    
    node.removeDisplayAttribute(ONCLICK);
    node.removeDisplayAttribute(ID_ATTR);
  }
}
