/*
 * Copyright 2013 Torindo Nesci.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.flamenk.article.manipulators;

import com.google.common.base.Preconditions;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

/**
 * A social network item such as Twitter button or Facebook like button.
 * The following Twitter share button <br/>
 * <tt>&lt;a
 * href="https://twitter.com/share"
 * class="twitter-share-button"
 * data-via="torindonesci"
 * data-lang="en"&gt;Tweet&lt;/a&gt;
 * </tt>
 * <br/>
 * Can be mapped to the social item using the following code:
 * <pre>
 * SocialNetworkItem twitterShare =
 *            new SocialNetworkItem("a")
 *            .withAttribute("href", "https://twitter.com/share");
 * </pre>
 * or with
 * <pre>
 * SocialNetworkItem twitterShare =
 *            new SocialNetworkItem("a")
 *            .withAttribute("href", "https://twitter.com/share")
 *            .withAttribute("class", "twitter-share-button");
 * </pre>
 *
 * <br/><br/><b>This implementation is Thread Safe.</b>
 *
 * @author Torindo Nesci
 */
public class SocialNetworkItem {
  
  private final Map<String, String> mAttributes
          = new ConcurrentHashMap<String, String>();
  private final String mHtmlTag;
  
  /**
   * Create a new social network item.
   *
   * @param htmlTag The html tag which renders the social network item.
   */
  public SocialNetworkItem(String htmlTag) {
    Preconditions.checkNotNull(htmlTag);
    
    this.mHtmlTag = htmlTag;
  }
  
  /**
   * Adds the given attribute name and value to the attribute
   * of this social item.
   *
   * @param name The attribute name. For instance <tt>class</tt>.
   * @param value The attribute value. For instance <tt>fb-like</tt>
   * @return This social item.
   */
  public SocialNetworkItem withAttribute(String name, String value) {
    Preconditions.checkNotNull(name);
    Preconditions.checkNotNull(value);
    
    this.mAttributes.put(name, value);
    return this;
  }
  
  /**
   * Returns the tag name of this social item.
   *
   * @return The tag name.
   */
  public String getTagName() {
    return this.mHtmlTag;
  }
  
  /**
   * Returns the attributes of this social item.
   *
   * @return The attributes.
   */
  public Map<String, String> getAttributes() {
    return this.mAttributes;
  }
  
  @Override
  public int hashCode() {
    int hash = 7;
    hash = 53 * hash + (this.mAttributes != null ? this.mAttributes.hashCode() : 0);
    hash = 53 * hash + (this.mHtmlTag != null ? this.mHtmlTag.hashCode() : 0);
    return hash;
  }

  @Override
  public boolean equals(Object obj) {
    if (obj == null) {
      return false;
    }
    if (getClass() != obj.getClass()) {
      return false;
    }
    final SocialNetworkItem other = (SocialNetworkItem) obj;
    if (this.mAttributes != other.mAttributes
            && (this.mAttributes == null || !this.mAttributes.equals(other.mAttributes))) {
      return false;
    }
    if ((this.mHtmlTag == null)
            ? (other.mHtmlTag != null) : !this.mHtmlTag.equals(other.mHtmlTag)) {
      return false;
    }
    return true;
  }
 
}
