/*
 * Copyright 2013 Torindo Nesci.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.flamenk.dom;

import com.flamenk.TagConstants;
import com.flamenk.util.Consumer;
import static com.flamenk.util.NodeUtil.hasName;
import com.google.common.base.Optional;
import com.google.common.base.Preconditions;
import java.util.ArrayList;
import java.util.Deque;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import org.apache.commons.lang.StringEscapeUtils;
import org.jsoup.nodes.Document;
import org.jsoup.nodes.Node;

/**
 * A Html Document Tree with all nodes ranked as possible article.
 *
 * <br/><br/> <b>This implementation is not Thread Safe.</b>
 *
 * @author Torindo Nesci
 */
public class HtmlDocument {

  private Optional<String> mTitle;
  private HtmlNode mRoot;
  private HtmlNode mMaxRankNode;
  private HtmlNodeArticleRanker mNodeRanker =
          SimpleHtmlNodeArticleRanker.getInstance();
  private TextTokenizer mTextTokenizer =
          SimpleTextTokenizer.getInstance();

  /**
   * Creates a new {@link HtmlDocument}.
   *
   */
  HtmlDocument() {
  }

  /**
   * Parses the given Jsoup document.
   *
   * @param doc The Jsoup document which represent the plain dom.
   * @see org.jsoup.Jsoup
   */
  void parse(Document doc) {
    Preconditions.checkNotNull(doc);

    String title = doc.title();
    if (title == null || title.trim().isEmpty()) {
      this.mTitle = Optional.absent();
    } else {
      this.mTitle = Optional.of(title);
    }

    Node rootNode = null;
    for (Node node : doc.childNodes()) {
      if (TagConstants.HTML.equals(node.nodeName())) {
        rootNode = node;
      }
    }
    if (rootNode == null) {
      throw new IllegalArgumentException("The resource at "
              + doc.baseUri() + " is not a valid html document."
              + " Does not contain the body root tag.");
    }

    BuildTreeInfo info = new BuildTreeInfo();
    info = buildTree(rootNode, null, 0, info);
    this.mRoot = info.node;
    this.mMaxRankNode = info.nodeMaxRank;
  }

  private static class BuildTreeInfo {

    Deque<HtmlNode> nodes;
    HtmlNode node;
    int nodesBuilt;
    double maxRank = 0;
    HtmlNode nodeMaxRank;

    public BuildTreeInfo with(Deque<HtmlNode> htmlNodes, int htmlNodesBuilt) {
      this.nodes = htmlNodes;
      this.nodesBuilt = htmlNodesBuilt;

      return this;
    }

    public BuildTreeInfo with(HtmlNode htmlNode, int htmlNodesBuilt) {
      this.node = htmlNode;
      this.nodesBuilt = htmlNodesBuilt;

      return this;
    }
  }

  private BuildTreeInfo buildTrees(
          List<Node> nodes, HtmlNode parent, int nodeIdx, BuildTreeInfo info) {
    Deque<HtmlNode> htmlNodes = new LinkedList<HtmlNode>();
    
    int curNodeIdx = nodeIdx;
    for (Node node : nodes) {
      BuildTreeInfo htmlNodeInfo = buildTree(node, parent, curNodeIdx, info);
      HtmlNode htmlNode = htmlNodeInfo.node;
      curNodeIdx += htmlNodeInfo.nodesBuilt;
      htmlNodes.add(htmlNode);
    }

    int nodesBuilt = curNodeIdx - nodeIdx;
    return info.with(htmlNodes, nodesBuilt);
  }

  private BuildTreeInfo buildTree(
          Node node, HtmlNode parent, int nodeIdx, BuildTreeInfo info) {
    if (TagConstants.TEXT.equals(node.nodeName())) {
      HtmlNode textNode = buildTextNode(node, parent, nodeIdx);
      return info.with(textNode, 1);
    } else if (TagConstants.TEXTAREA.equals(node.nodeName())) {
      HtmlNode textareaNode = newHtmlNode(node, parent, nodeIdx);
      return info.with(textareaNode, 1);
    }
        
    List<Node> children = new ArrayList<Node>(node.childNodes().size());
    for (Node child : node.childNodes()) {
      if (skip(child)) {
        continue;
      }
      children.add(child);
    }
    
    HtmlNode htmlNode = newHtmlNode(node, parent, nodeIdx);
    BuildTreeInfo childrenInfo = buildTrees(children, htmlNode, nodeIdx + 1, info);
    HtmlNodeMetrics nodeMetrics = htmlNode.getMetrics();
    for (HtmlNode htmlChild : childrenInfo.nodes) {
      HtmlNodeMetrics childMetrics = htmlChild.getMetrics();
      nodeMetrics.add(childMetrics);
      
      int numChildTokens = getNumTrailingTags(htmlChild.getJsoupNode());
      int numChildDivTokens = 0;
      if (hasName(htmlChild, TagConstants.DIV)) {
        numChildDivTokens += 2;
      }
      nodeMetrics.addNumTokens(numChildTokens);
      nodeMetrics.addNumDivTokens(numChildDivTokens);
      
      calcAndSetChildMetrics(
              htmlNode, htmlChild, numChildTokens, numChildDivTokens);
    }

    checkAndSetMaxRankNode(htmlNode, info);
    
    return info.with(htmlNode, childrenInfo.nodesBuilt + 1);
  }

  private void calcAndSetChildMetrics(
          HtmlNode node,
          HtmlNode child,
          int numChildTokens,
          int numChildDivTokens) {

      if (hasName(child, TagConstants.DIV)
              || hasName(child, TagConstants.ARTICLE)) {
        HtmlNodeMetrics metrics = HtmlNodeMetrics.newCopy(child.getMetrics());
        metrics.addNumDivTokens(numChildDivTokens);
        metrics.addNumTokens(numChildTokens);
        node.addChildBlockMetrics(metrics);
      } else {
        HtmlNodeMetrics noBlockMetrics = node.getChildNoBlockMetrics();
        noBlockMetrics.add(child.getChildNoBlockMetrics());
        noBlockMetrics.addNumDivTokens(numChildDivTokens);
        noBlockMetrics.addNumTokens(numChildTokens);
        
        node.addAllChildBlockMetrics(child.getMutableChildBlockMetrics());
      }
  }
  
  private HtmlNode buildTextNode(Node node, HtmlNode parent, int nodeIdx) {
    HtmlNode htmlTextNode = newHtmlNode(node, parent, nodeIdx);
    String text = node.outerHtml();
    text = StringEscapeUtils.unescapeHtml(text).trim();
    int numTextTokens = 0;
    boolean isLinkText = false;
    boolean isListItemText = false;
    if (!text.isEmpty()) {
      List<HtmlNode> ancestors = new LinkedList<HtmlNode>();
      for (Optional<HtmlNode> ancestor = htmlTextNode.getParent();
              ancestor.isPresent();
              ancestor = ancestor.get().getParent()) {
        ancestors.add(ancestor.get());
      }

      String[] terms = this.mTextTokenizer.tokenize(text);
      numTextTokens = terms.length;

      for (String term : terms) {
        htmlTextNode.getMetrics().addTerm(term);
        for (HtmlNode ancestor : ancestors) {
          
          if (hasName(ancestor, TagConstants.A)) {
            isLinkText = true;
            break;
          } else if (hasName(ancestor, TagConstants.LI)) {
            isListItemText = true;
            break;
          }

        }
      }
    }

    HtmlNodeMetrics metrics = htmlTextNode.getMetrics();
    metrics.setNumTokens(numTextTokens);
    metrics.setNumTextTokens(numTextTokens);
    metrics.setNumDivTokens(0);
    metrics.setNumTextLinkTokens(0);
    metrics.setNumTextListItemTokens(0);
    if (isLinkText) {
      metrics.setNumTextLinkTokens(numTextTokens);
    } else if (isListItemText) {
      metrics.setNumTextListItemTokens(numTextTokens);
    }

    htmlTextNode.getChildNoBlockMetrics().add(metrics);
    return htmlTextNode;
  }
  
  private void checkAndSetMaxRankNode(HtmlNode htmlNode, BuildTreeInfo info) {
    double nodeRank = this.mNodeRanker.rank(htmlNode, this.mTextTokenizer);
    if (info.nodeMaxRank == null || nodeRank > info.maxRank) {
      info.nodeMaxRank = htmlNode;
      info.maxRank = nodeRank;
    }
  }
  
  private HtmlNode newHtmlNode(Node node, HtmlNode parent, int nodeIdx) {
    HtmlNode htmlNode;
    if (parent == null) {
      htmlNode = new HtmlNode(node, nodeIdx, this);
    } else {
      htmlNode = new HtmlNode(node, parent, nodeIdx, this);
    }
    return htmlNode;
  }

  private int getNumTrailingTags(Node node) {
    if (node.nodeName().startsWith("#")) {
      return 0;
    }
    if (node.childNodeSize() > 1) { //A node might contain an empty text node
      return 2;
    }
    String html = node.outerHtml();
    String closingTag = "</" + node.nodeName() + ">";
    if (html.lastIndexOf(closingTag) == (html.length() - closingTag.length())) {
      return 2;
    } else {
      return 1;
    }
  }

  private boolean skip(Node node) {
    return TagConstants.SCRIPT.equals(node.nodeName())
            || TagConstants.NOSCRIPT.equals(node.nodeName())
            || TagConstants.DATA.equals(node.nodeName())
            || TagConstants.COMMENT.equals(node.nodeName())
            || TagConstants.DOCTYPE.equals(node.nodeName())
            || TagConstants.HEAD.equals(node.nodeName())
            || TagConstants.STYLE.equals(node.nodeName());
  }

  /**
   * Sets the text tokenizer.
   *
   * @param textTokenizer The text tokenizer.
   */
  void setTextTokenizer(TextTokenizer textTokenizer) {
    Preconditions.checkNotNull(textTokenizer);

    this.mTextTokenizer = textTokenizer;
  }

  /**
   * Sets the node article ranker.
   *
   * @param nodeRanker The html node article ranker.
   */
  void setNodeRanker(HtmlNodeArticleRanker nodeRanker) {
    Preconditions.checkNotNull(nodeRanker);

    this.mNodeRanker = nodeRanker;
  }

  /**
   * Returns the node with the max article rank.
   *
   * @return The node with max article rank.
   * @throws IllegalStateException If the document was not parsed.
   */
  public Optional<HtmlNode> getNodeWithMaxRank() {
    Preconditions.checkState(this.mRoot != null, "No document parsed.");

    if (this.mMaxRankNode == null) {
      return Optional.absent();
    }
    return Optional.of(this.mMaxRankNode);
  }

  /**
   * Returns the root node.
   *
   * @return The root node.
   * @throws IllegalStateException If the document was not parsed.
   */
  public HtmlNode getRootNode() {
    Preconditions.checkState(this.mRoot != null, "No document parsed.");

    return this.mRoot;
  }

  /**
   * Returns the html document's tile contained in the head section.
   *
   * @return The document's title.
   */
  public Optional<String> getTitle() {
    return this.mTitle;
  }

  /**
   * Consumes all the html document nodes in the range.
   *
   * @param range The range in which consume the nodes.
   * @param consumer The consumer.
   * @throws IllegalStateException If the document was not parsed.
   * @see Consumer
   */
  public void consumeNodesInRange(HtmlNodeRange range, Consumer<HtmlNode> consumer) {
    Preconditions.checkNotNull(range);
    Preconditions.checkNotNull(consumer);
    Preconditions.checkState(this.mRoot != null, "No document parsed.");

    Deque<HtmlNode> stack = new LinkedList<HtmlNode>();
    stack.push(this.mRoot);
    while (!stack.isEmpty()) {
      HtmlNode currNode = stack.pop();
      if (range.isNodeInRange(currNode)) {
        consumer.consume(currNode);
      }
      Iterator<HtmlNode> nodeIt = currNode.getChildren().descendingIterator();
      while (nodeIt.hasNext()) {
        HtmlNode child = nodeIt.next();
        stack.push(child);
      }
    }
  }

  /**
   * Returns all the nodes with the given name in the given node range.
   *
   * @param nodeName The node name. e.g. <i> #text, div, a, img, etc.</i>
   * @param range The node range.
   * @return The list of nodes with the given name in the given range
   * @throws IllegalStateException If the document was not parsed.
   */
  public List<HtmlNode> getAllNodesByName(String nodeName, HtmlNodeRange range) {
    Preconditions.checkNotNull(nodeName);
    Preconditions.checkState(this.mRoot != null, "No document parsed.");

    List<HtmlNode> nodes = new LinkedList<HtmlNode>();
    Deque<HtmlNode> stack = new LinkedList<HtmlNode>();
    stack.push(this.mRoot);
    while (!stack.isEmpty()) {
      HtmlNode currNode = stack.pop();
      if (range.isNodeInRange(currNode)
              && currNode.getName().equals(nodeName)) {
        nodes.add(currNode);
      }
      Iterator<HtmlNode> nodeIt = currNode.getChildren().descendingIterator();
      while (nodeIt.hasNext()) {
        HtmlNode child = nodeIt.next();
        stack.push(child);
      }
    }

    return nodes;
  }
    
  /**
   * Returns the first node with the given name in the given node range.
   *
   * @param nodeName The node name. e.g. <i> #text, div, a, img, etc.</i>
   * @param range The node range.
   * @return The first node with the given name if it exists.
   * @throws IllegalStateException If the document was not parsed.
   */
  public Optional<HtmlNode> getFirstNodeByName(String nodeName, HtmlNodeRange range) {
    Preconditions.checkNotNull(nodeName);
    Preconditions.checkNotNull(range);
    Preconditions.checkState(this.mRoot != null, "No document parsed.");

    Deque<HtmlNode> stack = new LinkedList<HtmlNode>();
    stack.push(this.mRoot);
    while (!stack.isEmpty()) {
      HtmlNode currNode = stack.pop();
      if (range.isNodeInRange(currNode)
              && currNode.getName().equals(nodeName)) {
        return Optional.of(currNode);
      }
      Iterator<HtmlNode> nodeIt = currNode.getChildren().descendingIterator();
      while (nodeIt.hasNext()) {
        HtmlNode child = nodeIt.next();
        stack.push(child);
      }
    }
    return Optional.absent();
  }
  
  /**
   * Returns the last node with the given name in the given node range.
   *
   * @param nodeName The node name. e.g. <i> #text, div, a, img, etc.</i>
   * @param range The node range.
   * @return The last node with the given name if it exists.
   * @throws IllegalStateException If the document was not parsed.
   */
  public Optional<HtmlNode> getLastNodeByName(String nodeName, HtmlNodeRange range) {
    Preconditions.checkNotNull(nodeName);
    Preconditions.checkNotNull(range);
    Preconditions.checkState(this.mRoot != null, "No document parsed.");

    HtmlNode node = null;
    Deque<HtmlNode> stack = new LinkedList<HtmlNode>();
    stack.push(this.mRoot);
    while (!stack.isEmpty()) {
      HtmlNode currNode = stack.pop();
      if (range.isNodeInRange(currNode)
              && currNode.getName().equals(nodeName)) {
        node = currNode;
      }
      Iterator<HtmlNode> currNodeIt = currNode.getChildren().descendingIterator();
      while (currNodeIt.hasNext()) {
        HtmlNode child = currNodeIt.next();
        stack.push(child);
      }
    }

    if (node == null) {
      return Optional.absent();
    }
    return Optional.of(node);
  }
}
