/*
 * Copyright 2013 Torindo Nesci.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.flamenk.dom;

import com.google.common.base.Preconditions;
import java.io.IOException;
import java.net.URL;
import org.jsoup.Jsoup;
import org.jsoup.nodes.Document;

/**
 * A HtmlDocument builder.
 *
 * <br/><br/><b>This implementation is Thread Safe.</b>
 *
 * @author Torindo Nesci
 */
public final class HtmlDocumentBuilder {

  /**
   * A html document wrapper.
   *
   * <br/><br/><b>This implementation is Thread Safe.</b>
   *
   * @author Torindo Nesci
   */
  public static final class HtmlDocumentWrapper {
    private final HtmlDocument mHtmlDoc;
    private final long mStartProcessingTime;
    
    private HtmlDocumentWrapper(HtmlDocument htmlDoc, long startProcessingTime) {
      this.mHtmlDoc = htmlDoc;
      this.mStartProcessingTime = startProcessingTime;
    }
    
    /**
     * Return the html document.
     *
     * @return The html document.
     */
    public HtmlDocument getHtmlDocument() {
      return this.mHtmlDoc;
    }
    
    /**
     * Return the start processing time in milliseconds of Flamenk parsing.
     * The time begins when the html has been already downloaded,
     * therefore it does not takes into account the network time.
     *
     * @return The start processing time in milliseconds.
     */
    public long getStartProcessingTime() {
      return this.mStartProcessingTime;
    }
  }
  
  private static final HtmlDocumentBuilder INSTANCE = new HtmlDocumentBuilder();

  private HtmlDocumentBuilder() {
  }

  /**
   * Returns the singleton instance of {@link HtmlDocumentBuilder}.
   *
   * @return The singleton instance of {@link HtmlDocumentBuilder}.
   */
  public static HtmlDocumentBuilder getInstance() {
    return INSTANCE;
  }

  /**
   * Builds the {@link HtmlDocument}.
   *
   * @param nodeRanker The html node article ranker.
   * @param textTokenizer The text tokenizer.
   * @param url The url to build the html document from.
   * @param msTimeout The http connection timeout in milliseconds.
   * @return The {@link HtmlDocument}.
   * @throws IOException If the connection cannot be established with the given
   * url.
   */
  public HtmlDocumentWrapper build(
          HtmlNodeArticleRanker nodeRanker,
          TextTokenizer textTokenizer,
          URL url,
          int msTimeout) throws IOException {
    Preconditions.checkNotNull(nodeRanker);
    Preconditions.checkNotNull(textTokenizer);
    Preconditions.checkNotNull(url);
    Preconditions.checkArgument(msTimeout >= 0);

    Document jsoupDoc = Jsoup.parse(url, msTimeout);
    
    long startProcTime = System.currentTimeMillis();
    HtmlDocument htmlDoc = new HtmlDocument();
    htmlDoc.setNodeRanker(nodeRanker);
    htmlDoc.setTextTokenizer(textTokenizer);
    htmlDoc.parse(jsoupDoc);

    return new HtmlDocumentWrapper(htmlDoc, startProcTime);
  }
}
