/*
 * Copyright 2013 Torindo Nesci.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.flamenk.dom;

import com.google.common.base.Optional;
import com.google.common.base.Preconditions;

/**
 * A html node range.
 * The range must have at least either the start or end node.
 *
 * <br/><br/> <b>This implementation is not Thread Safe.</b>
 *
 * @author Torindo Nesci
 */
public class HtmlNodeRange {
  
  private HtmlNode mStartNode;
  private HtmlNode mEndNode;
  private boolean mStartInclusive = false;
  private boolean mEndInclusive = false;;
  
  /**
   * Creates a new node range.
   */
  public HtmlNodeRange() {
  
  }
  
  /**
   * Creates a new node range with start and end node.
   * The start and end nodes are exclusive by default.
   *
   * @param startNode The range start node. It's excluded by default.
   * @param endNode The range end node. It's excluded by default.
   * @throws IllegalArgumentException If the start and end nodes do not
   * belong to the same html document.
   */
  public HtmlNodeRange(HtmlNode startNode, HtmlNode endNode) {
    Preconditions.checkNotNull(startNode);
    Preconditions.checkNotNull(endNode);
    Preconditions.checkArgument(
            startNode.getHtmlDocument() == endNode.getHtmlDocument());
    
    this.mStartNode = startNode;
    this.mEndNode = endNode;
  }
  
  /**
   * Returns the start node of this range if exists.
   *
   * @return The start node.
   */
  public Optional<HtmlNode> getStartNode() {
    return optionalOf(this.mStartNode);
  }
  
  /**
   * Returns the end node of this range is exists.
   *
   * @return The end node.
   */
  public Optional<HtmlNode> getEndNode() {
    return optionalOf(this.mEndNode);
  }
  
  private <T> Optional<T> optionalOf(T obj) {
    if (obj == null) {
      return Optional.absent();
    }
    return Optional.of(obj);
  }
  
  /**
   * Sets the included start node of this range.
   * The node is part of the range.
   *
   * @param node The start node.
   * @return This {@link HtmlNodeRange}.
   * @throws IllegalArgumentException If the start and end nodes do not
   * belong to the same html document.
   */
  public HtmlNodeRange withIncludedStartNode(HtmlNode node) {
    Preconditions.checkNotNull(node);
    if (this.mEndNode != null) {
      Preconditions.checkArgument(
              node.getHtmlDocument() == this.mEndNode.getHtmlDocument());
    }
    
    this.mStartNode = node;
    this.mStartInclusive = true;
    return this;
  }
  
  /**
   * Sets the included end node of this range.
   * The node is part of the range.
   *
   * @param node The end node.
   * @return This {@link HtmlNodeRange}.
   * @throws IllegalArgumentException If the start and end nodes do not
   * belong to the same html document.
   */
  public HtmlNodeRange withIncludedEndNode(HtmlNode node) {
    Preconditions.checkNotNull(node);
    if (this.mStartNode != null) {
      Preconditions.checkArgument(
              node.getHtmlDocument() == this.mStartNode.getHtmlDocument());
    }
    
    this.mEndNode = node;
    this.mEndInclusive = true;
    return this;
  }
  
  /**
   * Sets the excluded start node of this range.
   * The node is not part of the range.
   *
   * @param node The start node.
   * @return This {@link HtmlNodeRange}.
   * @throws IllegalArgumentException If the start and end nodes do not
   * belong to the same html document.
   */
  public HtmlNodeRange withExcludedStartNode(HtmlNode node) {
    Preconditions.checkNotNull(node);
    if (this.mEndNode != null) {
      Preconditions.checkArgument(
              node.getHtmlDocument() == this.mEndNode.getHtmlDocument());
    }
    
    this.mStartNode = node;
    this.mStartInclusive = false;
    return this;
  }
  
  /**
   * Sets the excluded end node of this range.
   * The node is not part of the range.
   *
   * @param node The end node.
   * @return This {@link HtmlNodeRange}.
   * @throws IllegalArgumentException If the start and end nodes do not
   * belong to the same html document.
   */
  public HtmlNodeRange withExcludedEndNode(HtmlNode node) {
    Preconditions.checkNotNull(node);
    if (this.mStartNode != null) {
      Preconditions.checkArgument(
              node.getHtmlDocument() == this.mStartNode.getHtmlDocument());
    }
    
    this.mEndNode = node;
    this.mEndInclusive = false;
    return this;
  }
  
  /**
   * Sets the start node as included, it's part of the range.
   *
   * @return This {@link HtmlNodeRange}.
   * @throws IllegalStateException If the start node is not set.
   */
  public HtmlNodeRange withStartIncluded() {
    Preconditions.checkState(this.mStartNode != null);
    
    this.mStartInclusive = true;
    return this;
  }

  /**
   * Sets the end node as included, it's part of the range.
   *
   * @return This {@link HtmlNodeRange}.
   * @throws IllegalStateException If the end node is not set.
   */
  public HtmlNodeRange withEndIncluded() {
    Preconditions.checkState(this.mEndNode != null);
    
    this.mEndInclusive = true;
    return this;
  }
  
  /**
   * Sets the start node as not included, it's not part of the range.
   *
   * @return This {@link HtmlNodeRange}.
   * @throws IllegalStateException If the start node is not set.
   */
  public HtmlNodeRange withStartExcluded() {
    Preconditions.checkState(this.mStartNode != null);
    
    this.mStartInclusive = false;
    return this;
  }

  /**
   * Sets the end node as not included, it's not part of the range.
   *
   * @return This {@link HtmlNodeRange}.
   * @throws IllegalStateException If the end node is not set.
   */
  public HtmlNodeRange withEndExcluded() {
    Preconditions.checkState(this.mEndNode != null);
    
    this.mEndInclusive = false;
    return this;
  }
  
  /**
   * Tests if the given node is in this range.
   *
   * @param node The html node.
   * @return <code>true</code> if the node is in this range,
   * <code>false</code> otherwise.
   * @throws IllegalStateException If neither the start node
   * nor the end node is set.
   */
  public boolean isNodeInRange(HtmlNode node) {
    Preconditions.checkNotNull(node);
    Preconditions.checkState(
            this.mStartNode != null || this.mEndNode != null);
    
    boolean afterStart =
            this.mStartNode == null || node.isAfter(this.mStartNode);
    boolean beforeEnd =
            this.mEndNode == null || node.isBefore(this.mEndNode);
    
    if (afterStart && beforeEnd) {
      return true;
    }
    if (this.mStartInclusive && node == this.mStartNode) {
      return true;
    }
    if (this.mEndInclusive && node == this.mEndNode) {
      return true;
    }
    
    return false;
  }
  
  /**
   * Tests if the given node is before this range.
   *
   * @param node The html node.
   * @return <code>true</code> if the node is before this range,
   * <code>false</code> otherwise.
   * @throws IllegalStateException If neither the start node
   * nor the end node is set.
   */
  public boolean isNodeBeforeRange(HtmlNode node) {
    Preconditions.checkNotNull(node);
    Preconditions.checkState(
            this.mStartNode != null || this.mEndNode != null);
    
    boolean beforeStart =
            this.mStartNode != null && node.isBefore(this.mStartNode);
    if (beforeStart) {
      return true;
    }
    if (!this.mStartInclusive && node == this.mStartNode) {
      return true;
    }
 
    return false;
  }
  
  /**
   * Tests if the given node is after this range.
   *
   * @param node The html node.
   * @return <code>true</code> if the node is after this range,
   * <code>false</code> otherwise.
   * @throws IllegalStateException If neither the start node
   * nor the end node is set.
   */
  public boolean isNodeAfterRange(HtmlNode node) {
    Preconditions.checkNotNull(node);
    Preconditions.checkState(
            this.mStartNode != null || this.mEndNode != null);
    
     boolean afterEnd =
            this.mEndNode != null && node.isAfter(this.mEndNode);
     
    if (afterEnd) {
      return true;
    }
    if (!this.mEndInclusive && node == this.mEndNode) {
      return true;
    }
 
    return false;
  }
  
}
