/*
 * Copyright 2013 Torindo Nesci.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.flamenk.example;

import com.flamenk.Flamenk;
import com.flamenk.article.Article;
import com.google.common.base.Optional;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.UnsupportedEncodingException;
import java.net.URL;
import java.util.LinkedList;
import java.util.List;
import java.util.Scanner;

/**
 * A simple Flamenk usage example.
 *
 * @author Torindo Nesci
 */
public final class SimpleExampleMain {

  private static final String OUTPUT_DIR = "target" + File.separatorChar + "exampleout";
  private static final String URLS_FILENAME = "/urls.txt";

  private SimpleExampleMain() {
  }

  public static void main(String[] argv) throws IOException {
    System.out.println("Executing " + SimpleExampleMain.class.getSimpleName());
    try {
      File f = new File(OUTPUT_DIR);
      boolean created = f.mkdirs();
      if (!created) {
        System.out.println("Warning: Cannot create dir " + OUTPUT_DIR);
      }
    } catch (Exception ex) {
      System.out.println("Cannot create dir " + OUTPUT_DIR + ": " + ex.getMessage());
      System.exit(-1);
    }
    List<URL> urls = readUrls(URLS_FILENAME);
    List<Article> articles = extractArticles(urls);
    writeArticles(articles, OUTPUT_DIR);
  }

  private static void writeArticles(List<Article> articles, String outputDir)
          throws IOException {
    int count = 0;
    for (Article article : articles) {
      String filename = outputDir + File.separatorChar + count + ".html";
      writeArticle(article, filename);
      count++;
    }

  }

  private static void writeArticle(Article article, String filename)
          throws  IOException {
    Optional<String> title = article.getTitle();
    Optional<String> img = article.getMainImageUrl();
    Optional<String> body = article.getBody();
    
    String headerStr =
            "<html><head>"
            + "<meta "
            + "http-equiv=\"Content-Type\" "
            + "content=\"text/html; "
            + "charset=utf-8\"/>"
            + "</head><body>";
    String originalLinkStr =
            "<a href=\"" + article.getUrl().toString() + "\">"
            + "original page"
            + "</a>"
            + "<br/><br/>";
    byte[] header = str2Bytes(headerStr);
    byte[] originalLink = str2Bytes(originalLinkStr);
    byte[] footer = str2Bytes("</body></html>");

    File f = new File(filename);
    FileOutputStream fos = new FileOutputStream(f);
    try {
      fos.write(header);
      fos.write(originalLink);
      if (img.isPresent()) {
        String imgStr =
                "<img src=\"" + img.get() + "\" "
                + "style=\"border:solid 3px black\"/>"
                + "<br/><br/>";
        fos.write(str2Bytes(imgStr));
      }
      if (title.isPresent()) {
        String titleStr =  "<h1>" + title.get() + "</h1>";
        fos.write(str2Bytes(titleStr));
      }
      if (body.isPresent()) {
        fos.write(str2Bytes(body.get()));
      }
      fos.write(footer);
      System.out.println("wrote file " + filename);
    } finally {
      if (fos != null) {
        fos.close();
      }
    }

  }

  private static List<Article> extractArticles(List<URL> urls) {
    List<Article> articles = new LinkedList<Article>();
    for (URL url : urls) {
      try {
        System.out.println("Extracting article @ " + url + " ...");
        Optional<Article> article = Flamenk.extractArticle(url);
        if (article.isPresent()) {
          System.out.println("took " + article.get().getProcessingTime() + "ms.");
          articles.add(article.get());
        } else {
          System.out.println("Article not found @ " + url);
        }
      } catch (IOException ex) {
        System.out.println("Couldn't extract article from "
                + url + ": " + ex.getMessage());
      }
    }

    return articles;
  }

  private static List<URL> readUrls(String filename)
          throws IOException {
    InputStream is = SimpleExampleMain.class.getResourceAsStream(filename);
    try {
      Scanner scanner = new Scanner(is);
      List<URL> urls = new LinkedList<URL>();
      while (scanner.hasNextLine()) {
        String url = scanner.nextLine();
        if (url.startsWith("#") || url.trim().isEmpty()) {
          continue;
        }
        urls.add(new URL(url));
      }
      return urls;
    } finally {
      if (is != null) {
        is.close();
      }
    }
  }
  
  private static byte[] str2Bytes(String str)
          throws UnsupportedEncodingException {
    return str.getBytes("UTF8");
  }
}
