package com.flamenk.histogram;

import com.google.common.base.Preconditions;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

/**
 * A html tokens histogram.
 *
 * <br/><br/> <b>This implementation is not Thread Safe.</b>
 *
 * @author Torindo Nesci
 */
public class Histogram implements Iterable<HistogramBucket> {
  
  private List<HistogramBucket> mBuckets;
  
  /**
   * Creates a new histogram.
   *
   * @param nbuckets The expected number of buckets.
   * It's not mandatory to have exactly <code>nbuckets</code> buckets.
   */
  Histogram(int nbuckets) {
    Preconditions.checkArgument(nbuckets > 0);
    
    this.mBuckets = new ArrayList<HistogramBucket>(nbuckets);
  }
  
  /**
   * Adds a bucket to this histogram.
   *
   * @param bucket The histogram bucket.
   */
  void addBucket(HistogramBucket bucket) {
    Preconditions.checkNotNull(bucket);
    
    this.mBuckets.add(bucket);
  }

  /**
   * Returns the bucket at the specified position in the histogram.
   *
   * @param idx The bucket's index.
   * @return The bucket at the specified position in the histogram.
   */
  public HistogramBucket getBucketAt(int idx) {
    Preconditions.checkPositionIndex(idx, this.mBuckets.size());
    
    return this.mBuckets.get(idx);
  }
  
  /**
   * Returns the number of buckets contained in this histogram.
   *
   * @return The number of buckets.
   */
  public int size() {
    return this.mBuckets.size();
  }
  
  /**
   * Returns an iterator over the elements in this histogram
   * in proper sequence.
   *
   * @return The buckets iterator.
   */
  public Iterator<HistogramBucket> iterator() {
    return this.mBuckets.iterator();
  }
  
}
