package com.flamenk.histogram;

import com.flamenk.AlgorithmConfiguration;
import com.google.common.base.Preconditions;
import java.util.List;

/**
 *
 * A histogram builder.
 *
 * <br/><br/> <b>This implementation is Thread Safe.</b>
 *
 * @author Torindo Nesci
 */
public final class HistogramBuilder {

  private static final HistogramBuilder INSTANCE = new HistogramBuilder();
  
  private HistogramBuilder() {
  
  }
  
  /**
   * Returns the singleton instance of the histogram builder.
   *
   * @return The histogram builder.
   */
  public static HistogramBuilder getInstance() {
    return INSTANCE;
  }
  
  /**
   * Builds the histogram given the list of tokens
   * using {@link SimpleTokenRanker} and {@link AlgorithmConfiguration#getDefault()}.
   *
   * @param tokens The list of tokens.
   * @return The histogram.
   */
  public Histogram build(List<Token> tokens) {
    return build(
            tokens,
            SimpleTokenRanker.getInstance(),
            AlgorithmConfiguration.getDefault());
  }
  
  /**
   * Builds the histogram given the list of tokens.
   *
   * @param tokens The list of tokens.
   * @param ranker The token ranker
   * @param conf The configuration parameters.
   * @return The histogram.
   */
  public Histogram build(List<Token> tokens, TokenRanker ranker, AlgorithmConfiguration conf) {
    Preconditions.checkNotNull(tokens);
    Preconditions.checkNotNull(ranker);
    Preconditions.checkArgument(tokens.size() > 0);
    
    int nbuckets = tokens.size() / conf.getBucketTokens();
    nbuckets += tokens.size() % conf.getBucketTokens() > 0 ? 1 : 0;

    Histogram hist = new Histogram(nbuckets);
    int count = 0;
    HistogramBucket bucket = new HistogramBucket().withTokenRanker(ranker);
    for (Token token : tokens) {
      if (count == conf.getBucketTokens()) {
        hist.addBucket(bucket);
        bucket.setSmoothValue(bucket.getValue());
        if (hist.size() > 1) {
          smoothBucket(hist, hist.size() - 2, conf);
        }
        count = 0;
        bucket = new HistogramBucket().withTokenRanker(ranker);
      }
      count++;
      bucket.addToken(token);
    }
    hist.addBucket(bucket);
    bucket.setSmoothValue(bucket.getValue());
    if (hist.size() > 1) {
      smoothBucket(hist, hist.size() - 2, conf);
    }
    return hist;
  }
  
  private void smoothBucket(Histogram hist, int bucketIdx, AlgorithmConfiguration conf) {
    if (bucketIdx < 2 || bucketIdx >= (hist.size() - 1)) {
      return;
    }
    HistogramBucket bucket = hist.getBucketAt(bucketIdx);
    if (bucket.getSmoothValue() < conf.getMinBucketTextRatio()) {
      double bucketPrevVal = hist.getBucketAt(bucketIdx - 1).getSmoothValue();
      double bucketNextVal = hist.getBucketAt(bucketIdx + 1).getSmoothValue();
      if (bucketPrevVal >= conf.getMinBucketTextRatio()
              && bucketNextVal >= conf.getMinBucketTextRatio()) {
        double smoothValue =
                bucket.getValue() + bucketPrevVal + bucketNextVal;
        smoothValue /= 3;
        bucket.setSmoothValue(smoothValue);
      }
      if (bucketPrevVal >= conf.getMinBucketTextRatio()
              && bucketNextVal >= conf.getMinBucketTextRatio()) {
        double smoothValue = Math.min(bucket.getSmoothValue() + 0.1D, 1.0D);
        bucket.setSmoothValue(smoothValue);
      }
    }
  }
}
